"""
Словарь неправильно лемматизируемых слов.

Например, вместо леммы 'основной'
выдается лемма 'основный'.

Также можно преобразовать одни леммы на другие.
Например, 'частенько' -> 'часто'.
"""


fixed_lemmas = {
    "арен": "арена",
    "бело-голубым": "бело-голубой",
    "блокшота": "блокшот",
    "большинстве": "большинство",
    "бортик": "борт",
    "буллитная": "буллитный",
    "буллиты": "буллит",
    "бумажка": "бумага",
    "бывает": "бывать",
    "бывало": "бывать",
    "бьемся": "биться",
    "везет": "везти",
    "вели": "везти",
    "вести": "везти",
    "во": "в",
    "вообще-то": "вообще",
    "впритирку": "рядом",
    "временной": "временный",
    "вспомним": "вспомнить",
    "вспомните": "вспомнить",
    "выжали": "выжать",
    "говорили": "говорить",
    "говоря": "говорить",
    "года": "год",
    "голов": "гол",
    "голова": "гол",
    "голы": "гол",
    "дай": "дать",
    "да-да": "да",
    "девчонка": "девочка",
    "деньга": "деньги",
    "длиннее": "длинный",
    "добивании": "добивание",
    "добиваний": "добивание",
    "добивания": "добивание",
    "должок": "долг",
    "домой": "дом",
    "думается": "думаться",
    "думали": "думать",
    "думаю": "думать",
    "жалко": "жаль",
    "желаем": "желать",
    "забиваем": "забивать",
    "запасный": "запасной",
    "забей": "забить",
    "завести": "завезти",
    "защ": "защитник",
    "звездочка": "звезда",
    "звоночек": "звонок",
    "зевнули": "зевнуть",
    "злее": "злой",
    "знали": "знать",
    "значит": "значить",
    "знаю": "знать",
    "зрителя": "зритель",
    "играем": "играть",
    "играй": "играть",
    "играл": "играть",
    "играли": "играть",
    "игралось": "играться",
    "исполнилось": "исполниться",
    "используйте": "использовать",
    "казалось": "казаться",
    "каков": "какой",
    "касается": "касаться",
    "книжка": "книга",
    "которого": "который",
    "красивее": "красивый",
    "крути": "крутить",
    "кулачок": "кулак",
    "лавочка": "лавка",
    "ленточка": "лента",
    "локоток": "локоть",
    "лошадка": "лошадь",
    "маленько": "немного",
    "мальчишка": "мальчик",
    "машинка": "машина",
    "минское": "минский",
    "минутах": "минута",
    "минутка": "минута",
    "минуть": "миновать",
    "млн": "миллион",
    "многих": "многие",
    "многому": "многое",
    "молодая": "молодой",
    "молодчик": "молодец",
    "моментик": "момент",
    "мост": "мостик",
    "мужик": "мужчина",
    "наверное": "наверно",
    "напомним": "напомнить",
    "нет-нет": "нет",
    "ничей": "ничья",
    "нужнее": "нужно",
    "нынче": "сейчас",
    "нюансов": "нюанс",
    "об": "о",
    "обратите": "обратить",
    "обращаем": "обращать",
    "общаетесь": "общаться",
    "овертайме": "овертайм",
    "один-единственный": "единственный",
    "ожидали": "ожидать",
    "основный": "основной",
    "отвести": "отвезти",
    "отдохнули": "отдохнуть",
    "отмечу": "отметить",
    "отрезками": "отрезок",
    "очень-очень": "очень",
    "очень-то": "очень",
    "очки": "очко",
    "ошибаюсь": "ошибаться",
    "плюсик": "плюс",
    "повезло": "повезти",
    "повести": "повезти",
    "повторимся": "повториться",
    "повторюсь": "повториться",
    "подведите": "подвести",
    "получилось": "получиться",
    "получше": "лучше",
    "помните": "помнить",
    "понимали": "понимать",
    "посмотрите": "посмотреть",
    "постарше": "старше",
    "потихонечку": "потихоньку",
    "предыдущий": "прошлый",
    "придется": "прийтись",
    "программка": "программа",
    "пройдет": "пройти",
    "проспали": "проспать",
    "прочтете": "прочитать",
    "прямик": "прямиком",
    "пускай": "пусть",
    "рада": "рад",
    "радует": "радовать",
    "разбираем": "разбирать",
    "размесятся": "размеситься",
    "разу": "раз",
    "рановато": "рано",
    "ребятам": "ребята",
    "ребятами": "ребята",
    "регулярок": "регулярка",
    "рижское": "рижский",
    "родный": "родной",
    "руб": "рубль",
    "рыбка": "нырок",
    "сами": "сам",
    "сейвами": "сейв",
    "сейвище": "сейв",
    "сейвова": "сейв",
    "сейвовый": "сейв",
    "секунд": "секунда",
    "силенка": "сила",
    "скажем": "сказать",
    "сложновато": "сложно",
    "смотрите": "смотреть",
    "со": "с",
    "сопернику": "соперник",
    "спрашивайте": "спрашивать",
    "среднее": "средний",
    "стараешься": "стараться",
    "стенка": "стена",
    "страничка": "страница",
    "ступенька": "ступень",
    "суметь": "смочь",
    "считаете": "считать",
    "сыграем": "сыграть",
    "сыграли": "сыграть",
    "сэйвах": "сейв",
    "сэйвов": "сейв",
    "сэйвова": "сейв",
    "таков": "такой",
    "типа": "тип",
    "того": "тот",
    "только-только": "только",
    "тоненький": "тонкий",
    "тотала": "тотал",
    "траффик": "трафик",
    "тут-то": "тут",
    "тыс": "тысяча",
    "ува": "увы",
    "уважаемые": "уважаемый",
    "устал": "устать",
    "усталь": "устать",
    "уфимка": "уфимец",
    "уходим": "уходить",
    "форчек": "форчекинг",
    "хайлайты": "хайлайт",
    "хозяйка": "хозяин",
    "хоккеистка": "хоккеист",
    "хорошенько": "хорошо",
    "хотелось": "хотеться",
    "цвета": "цвет",
    "целое": "целый",
    "частенько": "часто",
    "челленджа": "челлендж",
    "чемпионка": "чемпион",
    "чередует": "чередовать",
    "честной": "честный",
    "чувствовалось": "чувствоваться",
    "шажок": "шаг",
    "шайб": "шайба",
    "шарик": "шар",
    "шт": "штука",
    "date-го": "date",
    "org-арена": "арена",
}
